/*
 * Calling the rpc server
 *
 * Copyright 1988 Purdue Research Foundation
 */
#include "config.h" 
#ifdef NFS
#define MAX_PRIV	(IPPORT_RESERVED-1)
#define MIN_PRIV	(IPPORT_RESERVED/2)
/* this whole file disappears in a puff of smoke if we're not on something
 *  that does the NFS thang.
 */
#include <stdio.h>
#include <errno.h>
extern int errno;
#include <rpc/rpc.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/param.h>
#include <netdb.h>
#include <strings.h>
#include "rpcstuff.h"

/*
 * rpc_client --
 * the client side of the stuff to entomb files from NFS clients.  We
 * make an RPC call to the server, which is hopefully executing as
 * root, and can entomb the file for us.  We're given the absolute pathname
 * of the file that needs to be entombed, the name the entomb program was
 * invoked with, and the name of the person for whom to entomb the file, if
 * it's known.  We call the server with the name of the program, the file to
 * entomb, the user to entomb it for (or NULL if we don't know), the
 * invoking user's numeric uid, and a list of the invoking user's groups.
 */
void
rpc_client(host, progname, file, user)
	register char *host, *progname, *file, *user;
{
	static CLIENT *client;
	static struct timeval pertry_time, total_time;
	static struct hostent *hp;
	static struct sockaddr_in server_addr, clnt_addr;
	static enum clnt_stat clnt_stat;
	static int sock, status, i;
	static char *args[3];

	args[0] = progname;
	args[1] = file;
	args[2] = user;

#ifdef DEBUG
printf("rpc_client()\n");
printf("gethostbyname %s ...\n", host);
#endif

	if (NULL == (hp = gethostbyname(host))) {
		fprintf(stderr, "rpc_client: gethostbyname: can't find %s\n",
			host);
		return;
	}

	pertry_time.tv_sec = 6;
	pertry_time.tv_usec = 0;
	bcopy(hp->h_addr, &server_addr.sin_addr, hp->h_length);
	server_addr.sin_family = AF_INET;

	if ((sock = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)) < 0) {
		perror("socket:");
		return;
	}
	bzero((char *)& clnt_addr, sizeof clnt_addr);

	/*
	 * Bind the socket to a reserved port (requires "root" permission).
	 */
	clnt_addr.sin_family = AF_INET;
	clnt_addr.sin_addr.s_addr = INADDR_ANY;

	for (i = MAX_PRIV;;) {
		clnt_addr.sin_port = htons(i);
		if (-1 != bind(sock, (struct sockaddr *)& clnt_addr, sizeof
			clnt_addr))
			break;
		if (errno != EADDRINUSE && errno != EADDRNOTAVAIL) {
			perror("bind");
			exit(1);
		}
		if (MIN_PRIV > --i) {
			fprintf(stderr, "bind: all sockets in use\n");
			exit(1);
		}
	}
	
#ifdef DEBUG
printf("port %d\n", i);
printf("clntudp_create ....\n");
#endif

	if (NULL == (client = clntudp_create(&server_addr, (u_long) ENTOMBPROG,
		(u_long) ENTOMBVERS, pertry_time, & sock))) {
		clnt_pcreateerror("clntudp_create");
		return;
	}
	total_time.tv_sec = 40;
	total_time.tv_usec = 0;

#ifdef DEBUG
printf("clnt_call()\n");
#endif
	if (RPC_SUCCESS != clnt_call(client, (u_long) ENTOMBPROC, xdr_vec3,
		(char *)& args[0], xdr_void, (char *)0, total_time)) {
		clnt_perror(client, "clnt_call");
		return;
	}
	return;
}
#endif /*NFS*/
