/* define ERR to be whatever you want to be returned when there's an error */
#define ERR 0
#include <stdio.h>
#include <strings.h>
#include <sys/types.h>
#include <sys/stat.h>

/*
 * u_can_dink  -- determine whether the user with the given uid and
 * 	permitted groups can unlink the file with the given path.
 *	WARNING: this routine gives incorrect results when applied
 *	to directories
 */
int
u_can_dink(uid, ngroups, pigroups, path, fstatbufp)
	int uid, ngroups;
	int pigroups[];
	char *path;
	struct stat *fstatbufp;		/* statbuf for the file 'path'	*/
{
	auto int NOTUSED;
	struct stat dstatbuf; 		/* for and dirs			*/
	char *pch, *pchParentDir;
	int can_write;

	/*
	 * we make a special case for root - he can remove any file on
	 * the system.
	 */
	if (0 == uid) {
		return 1;
	}

	/*
	 * no one can remove files from a directory he doesn't have write
	 * permission on.
	 */
	if (NULL == (pch = rindex(path, '/'))) {
		/* if this isn't an absolute path name, we're supposed to
		 * find the file in the current directory
		 * WRONG !!
		 */
		pchParentDir = ".";
	} else {
		*pch = '\000';
		pchParentDir = path;
	}
	if ('\000' == *pchParentDir)
		pchParentDir = "/";
#ifdef S_IFLNK
	if (lstat(pchParentDir, & dstatbuf) < 0) {
#else
	if (stat(pchParentDir, & dstatbuf) < 0) {
#endif
		if (NULL != pch)
			*pch = '/';
		return ERR;
	}
	if (NULL != pch)
		*pch = '/';

	can_write = 0;
	if (uid == dstatbuf.st_uid && (dstatbuf.st_mode&0200)) {
		can_write = 1;
	}
	if (grp_member(ngroups, pigroups, (int) dstatbuf.st_gid) &&
		(dstatbuf.st_mode&020)) {
		can_write = 1;
	}
	if (uid != dstatbuf.st_uid && ! grp_member(ngroups, pigroups,
		(int) dstatbuf.st_gid) && (dstatbuf.st_mode&002)) {
		can_write = 1;
	}
	if (0 == can_write) {
		return 0;
	}

	/*
	 * if the directory is sticky, the file can only be removed if
	 * the user is the owner of the file or the owner of the directory.
	 * And has write permision on the directory.
	 */
	if (dstatbuf.st_mode&S_ISVTX) {
		if (uid != fstatbufp->st_uid && uid != dstatbuf.st_uid) {
			return 0;
		}
	}
	/*
	 * otherwise, the user can unlink the file
	 */
	return 1;
}

static int
grp_member(ngrps, pigroups, file_gid)
	int ngrps, *pigroups, file_gid;
{
	register int i;
	for (i = 0; i < ngrps; ++i)
		if (file_gid == pigroups[i])
			return 1;
	return 0;
}
