/*
 * entomb.c -- a program to run setuid root that renames a file to the tomb
 * 	and chown's it to the owner of the tomb.
 */
#include "config.h"
#include <stdio.h>
#include <strings.h>
#include <fstab.h>
#include <syslog.h>
#include <pwd.h>
#include <errno.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <sys/time.h>
#include <sys/param.h>
#include <sys/dir.h>
#include "tomb.h"
#include "fslist.h"
extern int errno;

extern void copy_to_tomb(), move_to_tomb(), diddle_path();
extern char *findtomb();
extern int setup_tomb();
extern char *which_host();

static char sbResult[MAXPATHLEN+MAXNAMLEN+1];
static char *pchProgname, *pchFilename, *pchUser;
static int Mode, Uid;
static int CharonUid = -1, CharonGid;

int
main(argc, argv)
	int argc;
	char **argv;
{
	struct stat statbuf;
	int what_to_do;
	int fcopy = 0;
	struct passwd *pwent;
	int sbigroups[NGROUPS], ngrps;
	char *pchFsName;
	char *pchMntPnt;

	(void) umask(077);
	Uid = getuid();

	if (2 != argc) {
		fprintf(stderr, "usage: entomb file\n");
		exit(1);
	}
	openlog("entomb", 0, LOG_DAEMON);

	pchProgname = argv[0];
	pchFilename = argv[1];

	if (0 == strcmp(pchProgname, "copy")) {
		fcopy = 1;
	}

	(void) setreuid(0, Uid);

#if defined DEBUG
	printf("lstat'ing %s\n", pchFilename);
#endif
#if defined S_IFLNK 
	if (lstat(pchFilename, & statbuf) < 0) {
		fprintf(stderr, "lstat: ");
		perror(pchFilename);
		exit(1);
	}
#else /* S_IFLNK */
	if (stat(pchFilename, & statbuf) < 0) {
		fprintf(stderr, "stat: ");
		perror(pchFilename);
		exit(1);
	}
#endif S_IFLNK

	/* UID -- we're still the user at this point	*/

	/* we'll change the mode of the entombed file later to this */
	Mode = statbuf.st_mode;

	/*
	 * see if file should be entombed - if not, simply exit.
	 */
	if (UNLINK == (what_to_do = checkf(& statbuf, fcopy))) {
		exit(0);
	}
	ngrps = getgroups(NGROUPS, sbigroups);
	if (-1 == ngrps) {
		perror("getgroups");
		exit(1);
	}

	if (! u_can_dink(Uid, ngrps, sbigroups, pchFilename, & statbuf)) {
		exit(0);
	}
#if defined DEBUG
	printf("u can dink\n");
#endif

	(void) setuid(0);
	/* see who owns the file; I'll see if we got anybody later */
	pwent = getpwuid((int) statbuf.st_uid);

#if defined(NFS)
	/*
	 * if this file is mounted on an NFS client, or if the file belongs
	 * to an unknown user, we take a different approach: get the absolute
	 * pathname and call a remote procedure (running as root) to do the
	 * entombing.  Since the remote procedure doesn't know our CWD, we
	 * have to find the absolute pathname to the file that's to be
	 * entombed.  We pass the name of the owner of the file if we have
	 * it, because that person may not have an account on the server.
	 * FsName and MntPnt are set by which_host to be the name of the
	 * filesystem in question on the server and it's mount point on
	 * this host, respectively.  So if mace:/bob/ShareFiles is mounted
	 * on locahost:/share and we try to entomb localhost:/share/foo,
	 * we need to send /bob10/ShareFiles/foo to the entombd; he can't
	 * know where we mounted the filesystem he exported to us.  We
	 * call getwd() as the user, since root can't do the necessary
	 * stuff over NFS.
	 */
	if (NFS_MAJ_DEV == major(statbuf.st_dev) || NULL == pwent) {
		if ('/' != *pchFilename) {
			(void) setreuid(0, Uid);
			if (NULL == getwd(sbResult)) {
				fprintf(stderr, "entomb: can't getwd: %s\n",
					sbResult);
				exit(1);
			}
			(void) setuid(0);
			(void) strcat(sbResult, "/");
			(void) strcat(sbResult, pchFilename);
			pchFilename = & sbResult[0];
		}
		pchUser = (NULL == pwent) ? "" : pwent->pw_name;
		if (COPY == what_to_do || COPY_UNLINK == what_to_do) {
			pchProgname = "copy";
		} else {
			pchProgname = "move";
		}
		{
			char *pch;
			if (NULL == (pch = which_host(& statbuf, & pchFsName, & pchMntPnt))) {
				fprintf(stderr, "can't find remote host\n");
				exit(1);
			}
			diddle_path(& pchFilename, pchFsName, pchMntPnt);
			rpc_client(pch, pchProgname, pchFilename, pchUser);
			exit(0);
		}
	}
#endif /*NFS*/
	/*
	 * if we don't have a server to call, it's an error not to know
	 * the name of the owner of the file.
	 */
	if (NULL == pwent) {
		fprintf(stderr, "entomb: can't find passwd entry for file owner\n");
		exit(1);

	}

	/*
	 *  This code is a little bit paranoid:  any files which you
	 *  delete which you do not own are put in their owner's tomb
	 *  directory.  If you find that you need this file restored,
	 *  you will have to have the owner restore it for you.
	 *  This simplifies a lot of the coding, but is arguably not the
	 *  optimal solution.  We may actually want to check the files
	 *  permissions and act accordingly...
	 */
	
	if (FAIL == setup_tomb(& statbuf)) {
		/* 
		 * tomb directory does not exist on this filesystem,
		 * which tells us not to entomb any files removed here.
		 */
		exit(1);
	}
	/*
	 * if charon's uid is unset, we haven't looked it up yet.
	 * We'll need it in copy or move_to_tomb, so do it now.
	 */
	if (-1 == CharonUid) {
		if (NULL == (pwent = getpwnam(TOMBID))) {
			fprintf(stderr, "can't get %s passwd entry\n", TOMBID);
			exit(1);
		}
		CharonUid = pwent->pw_uid;
		CharonGid = pwent->pw_gid;
	}

	if (COPY == what_to_do || COPY_UNLINK == what_to_do) {
		copy_to_tomb(pchFilename);
	} else {
		move_to_tomb(pchFilename);
	}
	exit(0);
}

/*
 * setup_tomb -- get login name and create a subdirectory in the tomb
 * 	directory for that user.  puts the pathname (with a trailing
 *	slash) into `sbResult'.  Returns 0 if all went well, FAIL if there
 *	was some error.
 */
static int
setup_tomb(statbuf) 
	struct stat *statbuf;
{
	static char *tomb;
	struct passwd *pwent;

	/*
	 * get the user's login, so we know what directory to
	 * entomb the file in...
	 */
	if (NULL == (pwent = getpwuid((int) statbuf->st_uid))) {
		if (NULL == pchUser) {
			/* not a severe error */
			return NULL;
		}
	} else {
		pchUser = pwent->pw_name;
	}

	/*
	 * now append the user name to the tomb directory
	 */
	if (NULL == (tomb = findtomb(statbuf)))
		return FAIL;
	(void) strcpy(sbResult, tomb);
	(void) strcat(sbResult, "/");
	(void) strcat (sbResult, pchUser);

#ifdef DEBUG
printf("creating tomb directory %s\n", sbResult);
#endif
	/*
 	 * we have to ignore signals, here, otherwise the user can
	 * interrupt between the mkdir and the chown.  This can cause
	 * the tomb to be owned by root.  Since preend runs as charon,
	 * it can't remove the tomb or anything in it.
	 */
	(void) signal(SIGINT, SIG_IGN);
	(void) signal(SIGQUIT, SIG_IGN);
	if (FAIL == mkdir(sbResult, 0777)) {
		if (EEXIST != errno) {
			perror("couldn't create your tomb directory");
			return NULL;
		}
	} else {
		if (NULL == (pwent = getpwnam(TOMBID))) {
			fprintf(stderr, "can't get %s passwd entry\n", TOMBID);
			exit(1);
		}
		CharonUid = pwent->pw_uid;
		CharonGid = pwent->pw_gid;
		if (FAIL == chown(sbResult, CharonUid, CharonGid)) {
			perror("chown");
			exit(1);
		}
		if (FAIL == chmod(sbResult, 0770)) {
			perror("chmod");
			exit(1);
		}
	}
	(void) signal(SIGINT, SIG_DFL);
	(void) signal(SIGQUIT, SIG_DFL);
	(void) strcat(sbResult, "/");
	return 0;
}

/*
 * checkf -- check to see if a file should be entombed, and how
 *	it should be entombed - copy, rename, copy & unlink,
 *	or just plain old unlink. Also need to check ownership
 *	of file vs. invoker of rm as well as count the links.
 *	Some of these conditions may force a copy to a different
 *	tomb, rather than a combination of links/unlinks/renames.
 *	These conditions should be handled last since they are not
 *	likely to be common occurrences.
 */
static int
checkf(statbuf, fcopy)
	struct stat *statbuf;
	int fcopy;
{
	/*
	 * Rename regular files ( copy them if this has been called
	 * via an open with O_TRUNCATE as indicated in copy argument )
	 */
	if (S_IFREG == (statbuf->st_mode & S_IFMT) && 1 == statbuf->st_nlink) {
		if (statbuf->st_size == 0 ) {
#ifdef DEBUG
printf("what to do: unlink\n");
#endif
			return UNLINK;
		} else {
#ifdef DEBUG
printf("what to do: %s\n", fcopy ? "copy" : "rename");
#endif
			return fcopy ? COPY : RENAME;
		}
	}

	/* 
	 *   Copy and unlink hard links which might 
	 *   not be owned by the remover.
	 */
	if (statbuf->st_nlink > 1) {
#ifdef DEBUG
printf("what to do: %s\n", fcopy ? "copy" : "copy_unlink");
#endif
		return fcopy ? COPY : COPY_UNLINK;
	}
	
	/* 
	 * Remove symbolic links
	 */
	if (S_IFLNK == (statbuf->st_mode & S_IFMT)) {
#ifdef DEBUG
printf("what to do: unlink\n");
#endif
		return UNLINK;
	}

	/* if it isn't any of the above, just unlink it */
#ifdef DEBUG
printf("what to do: unlink\n");
#endif
	return UNLINK;
}

/*
 * Findtomb - returns the path to the name of the tomb as it would exist
 * on the filesystem in which the file is being deleted, as determined from
 * the statbuf.  First try the easy way, by parsing the config file.  This
 * is done by FsList().  If that doesn't work, we probably shouldn't be here.
 */
char *
findtomb(pstatbuf)
	struct stat *pstatbuf;
{
	int i;

	/*
	 * we compare the device numbers from the statbuf of the file being
	 * entombed against those in the config file in an effort to determine
	 * the name of the filesystem to entomb the file on.  This can fail
	 * if the config file is out of date.
	 */
#if defined DEBUG
printf("calling fslist\n");
#endif
	FsList();
	for (i = 0; major(pstatbuf->st_dev) != sbiMajorDevs[i]
		|| minor(pstatbuf->st_dev) != sbiMinorDevs[i]; ++i) {
		if (-1 == sbiMajorDevs[i]) {
			return NULL;
		}
	}
#if defined DEBUG
printf("found %s\n", sbpchTombDirs[i]);
#endif
	return sbpchTombDirs[i];
}

static void
copy_to_tomb(name)
	char *name;
{
	int from, to;  				/* file descriptors */
	int nbyte;     				/* number of bytes read */
	char *pch, buf[BUFSIZ];
	struct stat statbuf2;
	struct timeval TV;
	struct timezone TZ;
	struct passwd *pwent;

#if defined DEBUG
printf("copy to tomb: %s\n", name);
#endif
	(void) gettimeofday(& TV, & TZ);
	/* stick the filename onto the tomb directory name */

	if (NULL == (pch = rindex(name, '/')))
		pch = name - 1;
	(void) strcat(sbResult, pch+1);

	pch = & sbResult[strlen(sbResult)];

	do {
		(void) sprintf(pch,"@%ld", TV.tv_sec);

	} while (((to = open(sbResult, O_WRONLY|O_CREAT|O_EXCL, Mode&0777)) < 0)
		&& (errno==EEXIST) && (TV.tv_sec++));
	if (to < 0) {
		perror(sbResult);
		exit(1);
	}

	(void) fchown(to, CharonUid, CharonGid);
	(void) fchmod(to, Mode&0777 | 0400);

	if (FAIL == (from = open(name, O_RDONLY, 0))) {
		perror(name);
	}

	while ((nbyte = read(from, buf, sizeof buf)) > 0) {
		(void) write(to, buf, nbyte);
	}
	if (-1 == nbyte) {
		perror("read");
	}
	(void) close(from);
	(void) close(to);
}

static void
move_to_tomb(name)
	char *name;
{
	char *pch;
	struct timeval TV;
	struct timezone TZ;

#if defined DEBUG
printf("move to tomb: %s\n", name);
#endif
	(void) gettimeofday(& TV, & TZ);
	/* stick the filename onto the tomb directory name */

	if (NULL == (pch = rindex(name, '/')))
		pch = name - 1;
	(void) strcat(sbResult, pch+1);

	pch = & sbResult[strlen(sbResult)];

	(void) setuid(0);

	/* loop until we find a name that doesn't exist already */
	for (;;) {
		errno = 0;
		(void) sprintf(pch,"@%ld", TV.tv_sec++);
		if (link(name, sbResult) == 0 || EEXIST != errno)
			break;
	}

	(void) chown(sbResult, CharonUid, CharonGid);
	(void) chmod(sbResult, Mode&0777 | 0400);

	/* and the unlink will be done by the user */
}

/*
 * diddle_path -- fiddle with the path so that we get a full path
 * 	to the file to be entombed *as on the host where the file
 *	is "real"*, not like what we get from getwd().  The diddled
 *	path is placed back in ppchPath, which is supposed to be
 *	big enough to hold the pathname.  For the example, a foreign
 *	host "mace" exports the filesystem "/bob" to us.  /bob has
 *	a subdirectory "ShareFiles"; on the local host we do
 *		local# mount mace:/bob/ShareFiles /share
 *	Then J. Random User tries to entomb local:/share/jru/foobie
 *	This function is called with Path = /share/jru/foobie, FsName
 *	= /bob/ShareFiles, and MntPnt = /share
 */
static void
diddle_path(ppchPath, pchFsName, pchMntPnt)
char **ppchPath, *pchFsName, *pchMntPnt;
{
	static char buf[MAXPATHLEN+MAXNAMLEN+1];

	/* copy in "/bob/ShareFiles"	*/
	(void) strcpy(buf, pchFsName);	

	/* append  "/jru/foobie"	*/
	(void) strcat(buf, (*ppchPath)+strlen(pchMntPnt));

	/* put it back in ppchPath 	*/
	(void) strcpy(*ppchPath, buf);
}
