#include "config.h"
#if defined NFS
/* if we're not on system w/ nfs, the whole file disappears  */
#include <stdio.h>
#include <strings.h>
#include <sys/types.h>
#include <sys/stat.h>
#if defined GETMOUNTENT
#include <sys/param.h>
#include <sys/mount.h>
#endif
#if defined GETMNTENT
#include <mntent.h>
#endif

/*
 * which_host: given a statbuf on a file, return the name of the host upon
 *	which the file's filesystem is mounted, or NULL if there's an error.
 *	If the file is not mounted on an NFS filesystem, "localhost" is 
 *	returned.  Also, the name of the filesystem is returned, as is the
 *	mount point.
 *	no error messages are printed, because they're all commented out.
 */
char *
which_host(pSB, ppchFsName, ppchMntPnt)
	struct stat *pSB;
	char **ppchFsName, **ppchMntPnt;
{
	char *pch, *pchDev = NULL;

#if defined GETMOUNTENT
	{
		struct fs_data fs;
		int i = 0;
		while (1 == getmountent(& i, & fs, 1)) {
			char *pch;

			if (pSB->st_dev == fs.fd_dev) {
				pchDev = fs.fd_devname;
				*ppchFsName = fs.fd_devname;
				if (NULL != (pch = index(*ppchFsName, ':')))
					*ppchFsName = pch + 1;
				*ppchMntPnt = fs.fd_path;
				break;
			}
		}
	}
#else /* not GETMOUNTENT */
	{
		struct stat SBdev;
		struct mntent *mntent;
		FILE *pF;
		if (NULL == (pF = setmntent(MOUNTED, "r"))) {
#if defined TEST
			fprintf(stderr, "can't open %s\n", MOUNTED);
#endif /* TEST */
			return NULL;
		}
		while (NULL != (mntent = getmntent(pF))) {
			if (-1 == stat(mntent->mnt_dir, & SBdev)) {
#if defined TEST
				perror(mntent->mnt_dir);
#endif /* TEST */
				endmntent(pF);
				return NULL;
			}
			if (SBdev.st_dev == pSB->st_dev) {
				char *pch;

				pchDev = mntent->mnt_fsname;
				*ppchFsName = mntent->mnt_fsname;
				if (NULL != (pch = index(*ppchFsName, ':')))
					*ppchFsName = pch + 1;
				*ppchMntPnt = mntent->mnt_dir;
				break;
			}
		}
		endmntent(pF);
	}
#endif  /* GETMOUNTENT */
	if (NULL == pchDev) {
#if defined TEST
		fprintf(stderr, "filesystem not found\n");
#endif
		return NULL;
	}
	if (NULL == (pch = index(pchDev, ':'))) {
		return "localhost";
	}
	*pch = '\000';
	return pchDev;
}

#if defined TEST
int
main(argc, argv)
	int argc;
	char **argv;
{
	struct stat statbuf;
	char *pch;

	if (1 == argc) {
		fprintf(stderr, "usage: %s file ...\n", argv[0]);
		exit(1);
	}
	while (NULL != *++argv) {
		if (-1 == stat(*argv, & statbuf)) {
			perror(*argv);
			continue;
		}
		if (NULL == (pch = which_host(& statbuf)))
			(void) fprintf(stderr, "which_host: error\n");
		else
			(void) puts(pch);
	}
}
#endif /* TEST */
#endif /* NFS */
