/*
 * entomb.c -- this was originally the same as ../entomb/entomb.c, but
 * 	it got to be too hairy, putting the server and clients in the same
 *	file.  So this is pretty much the same.
 */
#include "config.h"
#include <stdio.h>
#include <strings.h>
#include <fstab.h>
#include <pwd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <sys/time.h>
#include <sys/param.h>
#include <sys/dir.h>
#include <syslog.h>
#include "tomb.h"
#include "fslist.h"
#include "nastygram.h"
extern int errno;

extern void copy_to_tomb(), move_to_tomb();
extern char *hard_way(), *findtomb();
extern int setup_tomb();

static int CharonUid, CharonGid;
static char sbResult[MAXPATHLEN+MAXNAMLEN+1];
static char *pchProgname, *pchFilename, *pchUser;
static int Mode, Uid;

/* 
 * entombd -- called by an rpc server to entomb a file on a remote host.
 * 	The file was on a NFS filesystem, which supposedly is real to us.
 */
void
entombd(argv)
	char **argv;
{
	struct stat statbuf;
	int fcopy = 0;
	int status;
	static struct passwd *pwent;

	(void) umask(077);

	pchProgname = argv[0];
	pchFilename = argv[1];
	pchUser = argv[2];

#ifdef S_IFLNK 
	if (lstat(pchFilename, & statbuf) < 0) {
#else S_IFLNK
	if (stat(pchFilename, & statbuf) < 0) {
#endif S_IFLNK
		syslog(LOG_ERR, "stat: %s: %m", pchFilename);
		return;
	}
	/* we'll change the mode of the entombed file later to this */
	Mode = statbuf.st_mode;

	/*
	 * if we can find the owner of the file, we can entomb it for
	 * that person.
	 */
	if (0 == strlen(pchUser)) {
		if (NULL != (pwent = getpwuid(statbuf.st_uid))) {
			pchUser = pwent->pw_name;
		} else {
			/* then we have no idea for whom to entomb */
			syslog(LOG_ERR, "can't find user %d\n", statbuf.st_uid);
			return;
		}
	}

	/*  Get the entombing id - for evading quotas */

	if (NULL == (pwent = getpwnam(TOMBID))) {
		syslog(LOG_ERR, "can't get TOMBID entombing information");
		return;
	}
	CharonUid = pwent->pw_uid;
	CharonGid = pwent->pw_gid;
	/*
	 *  This code is a little bit paranoid:  any files which you
	 *  delete which you do not own are put in their owner's tomb
	 *  directory.  If you find that you need this file restored,
	 *  you will have to have the owner restore it for you.
	 *  This simplifies a lot of the coding, but is arguably not the
	 *  optimal solution.  We may actually want to check the files
	 *  permissions and act accordingly...
	 */
	
	if (FAIL == setup_tomb(& statbuf)) {
		/* 
		 * tomb directory does not exist on this filesystem,
		 * which tells us not to entomb any files removed here.
		 */
		return;
	}

	if (0 == strcmp("copy", pchProgname)) {
		copy_to_tomb(pchFilename, & statbuf);
	} else {
		move_to_tomb(pchFilename);
	}
	return;
}

/*
 * setup_tomb -- get login name and create a subdirectory in the tomb
 * 	directory for that user.  puts the pathname (with a trailing
 *	slash) into `sbResult'.  Returns 0 if all went well, FAIL if there
 *	was some error.
 */
static int
setup_tomb(statbufp) 
	struct stat *statbufp;
{
	static char *tomb;
	struct passwd *pwd;

	/*
	 * get the user's login, so we know what directory to
	 * entomb the file in...
	 */
	if (NULL == pchUser) {
		if (NULL == (pwd = getpwuid(statbufp->st_uid))) {
			syslog("can't find your passwd entry, %d", statbufp->st_uid);
			return FAIL;
		}
	} 
	if (NULL == (tomb = findtomb(statbufp))) {
		/* Tomb does not exist */
		return FAIL;
	}
	if (FAIL == access(tomb, F_OK)) {
		return FAIL;
	}
	(void) strcpy(sbResult, tomb);
	(void) strcat(sbResult, "/");
	(void) strcat (sbResult, pchUser);

	if (FAIL == access(sbResult, F_OK)) {
		if (FAIL == mkdir(sbResult, 0777)) {
			syslog(LOG_ERR, "mkdir: %s: %m", sbResult);
			return FAIL;
		}
		if (FAIL == chown(sbResult, CharonUid, CharonGid)) {
			syslog(LOG_ERR, "chown: %s: %m", sbResult);
			return FAIL;
		}
		if (FAIL == chmod(sbResult, 0770)) {
			syslog(LOG_ERR, "chmod: %s: %m", sbResult);
			return FAIL;
		}
	}
	(void) strcat(sbResult, "/");
	return 0;
}


/*
 * Findtomb - returns the path to the name of the tomb as it would exist
 * on the filesystem in which the file is being deleted, as determined from
 * the statbuf.  First try the easy way, by parsing the config file.  This
 * is done by FsList().  If that doesn't work, send a nastygram to the owner
 * of the tomb, and do it the hard way.
 */
char *
findtomb(pstatbuf)
	struct stat *pstatbuf;
{
	int i;

	/*
	 * we compare the device numbers from the statbuf of the file being
	 * entombed against those in the config file in an effort to determine
	 * the name of the filesystem to entomb the file on.  This can fail
	 * if the config file is out of date.
	 */

	FsList();
	for (i = 0; major(pstatbuf->st_dev) != sbiMajorDevs[i]
		|| minor(pstatbuf->st_dev) != sbiMinorDevs[i]; ++i) {
		if (-1 == sbiMajorDevs[i]) {
			return hard_way(pstatbuf);
		}
	}
	return sbpchTombDirs[i];
}

/*
 * hard_way -- find the tomb-directory path the hard way, by calling
 * getfsent() and stat()'ing the files that that returns to see if they
 * have the device number we're looking for.
 */
char *
hard_way(pstatbuf)
	struct stat *pstatbuf;
{
	char sb[MAXPATHLEN + MAXNAMLEN + 1];
	struct fstab *pfs;
	struct stat stat_buf;

	if (0 == setfsent()) {
		syslog(LOG_ERR, "can't setfsent()");
		exit(2);
	}
	while (NULL != (pfs = getfsent())) {
		if (FAIL == stat(pfs->fs_spec, & stat_buf))
			continue;
		if (stat_buf.st_rdev != pstatbuf->st_dev)
			continue;
		break;
	}
	if (NULL == pfs) {
		syslog(LOG_ERR, "can't getfsent() the right entry");
		return (char *)0;
	}
	(void) sprintf(sb, "%s/%s", pfs->fs_file, CRYPT);
	(void) endfsent();
	return sb;
}

static void
copy_to_tomb(name, pstatbuf)
	char *name;
	struct stat *pstatbuf;
{
	int from, to;  				/* file descriptors */
	int nbyte;     				/* number of bytes read */
	char *pch, buf[BUFSIZ];
	struct stat statbuf2;
	static struct timeval TV;
	static struct timezone TZ;

	(void) gettimeofday(& TV, & TZ);
	(void) strcat(sbResult, name);

	if (NULL == (pch = rindex(name, '/')))
		pch = name - 1;
	(void) strcat(sbResult, pch+1);

	pch = & sbResult[strlen(sbResult)];

	do {
		(void) sprintf(pch,"@%ld", TV.tv_sec);

	} while (((to = open(sbResult, O_WRONLY|O_CREAT|O_EXCL, Mode&0777)) < 0) &&
		  (errno==EEXIST) && (TV.tv_sec++));
	if (to < 0) {
		syslog(LOG_ERR, "sbResult: %m");
		return;
	}

	(void) fchown(to, CharonUid, CharonGid);
	(void) fchmod(to, Mode&0777 | 0400);

	if (FAIL == (from = open(name, O_RDONLY, 0600))) {
		syslog(LOG_ERR, "open: %s: %m", name);
	}
	(void) fstat(from, & statbuf2);

	if (pstatbuf->st_ino != statbuf2.st_ino ||
	    pstatbuf->st_dev != statbuf2.st_dev) {
		(void) close(to);
		(void) close(from);
		return;
	} 

	while ((nbyte = read(from, buf, sizeof buf)) > 0) {
		(void) write(to, buf, nbyte);
	}
	if (-1 == nbyte) {
		syslog(LOG_ERR,"read: %m");
	}
	(void) close(from);
	(void) close(to);
}

static void
move_to_tomb(name)
	char *name;
{
	char *pch;
	int status;
	static struct timeval TV;
	static struct timezone TZ;

	(void) gettimeofday(& TV, & TZ);
	/* stick the filename onto the tomb directory name */

	if (NULL == (pch = rindex(name, '/')))
		pch = name - 1;
	(void) strcat(sbResult, pch+1);

	pch = & sbResult[strlen(sbResult)];

	(void) setuid(0);

	/* loop until we find a name that doesn't exist already */
	for (;;) {
		errno = 0;
		(void) sprintf(pch,"@%ld", TV.tv_sec++);
		if (link(name, sbResult) == 0 || EEXIST != errno)
			break;
	}

	(void) chown(sbResult, CharonUid, CharonGid);
	(void) chmod(sbResult, Mode&0777 | 0400);

	/* and the unlink will be done by the user */
}
