/*
 * stuff for an rpc entomb server.  The error messages that I print go
 * 	to syslog.  Somebody thought it would be clever for the rpc routines
 *	to print messages to stderr internally.  I don't know where they'll
 *	go.  This whole file is #ifdef'd out if this host doesn't run NFS.
 */
#include "config.h"
#ifdef NFS
#ifndef lint
static char *rcsid="$Id:";
#endif

#include <stdio.h>
#include <errno.h>
#include <syslog.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/wait.h>
#include <rpc/rpc.h>
#include "rpcstuff.h"

static char **envp;

static int
server(rqstp, transp)
	struct svc_req *rqstp;
	SVCXPRT *transp;
{
	/* a vector to give to execve */
	static char *args[4];

	if (NULLPROC == rqstp->rq_proc) {
#ifdef DEBUG
		fprintf(stdout, "NULLPROC\n");
#endif
		if (! svc_sendreply(transp, xdr_void, (char *) 0)) {
			syslog(LOG_ERR, "can't reply to NULLPROC call");
			exit(1);
		}
		exit(0);
	}

	switch (rqstp->rq_proc) {
	case ENTOMBPROC:
#ifdef DEBUG
		fprintf(stdout, "ENTOMBPROC\n");
#endif
		if (ntohs((svc_getcaller(transp))->sin_port) >= IPPORT_RESERVED) {
			svcerr_weakauth(transp);
			return 0;
		}
		if (! svc_getargs(transp, xdr_vec3, args)) {
			svcerr_decode(transp);
			return 0;
		}
		args[4] = NULL;
		if (! verify_caller(transp, & args[0])) {
			svcerr_weakauth(transp);
			return 0;
		}

		entombd(args);
		if (! svc_sendreply(transp, xdr_void, (char *) 0)) {
			syslog(LOG_ERR, "can't reply to call");
			exit(1);
		}
		exit(0);
	default:
		svcerr_noproc(transp);
		exit(1);
	}
	/*NOTREACHED*/
}

int
main(argc, argv)
	int argc;
	char **argv;
{
	struct sockaddr_in addr;
	int len = sizeof(struct sockaddr_in);
	SVCXPRT *transp;

#ifdef DEBUG
	{
		int s;
		struct sockaddr_in daddr;
		int dlen = sizeof(struct sockaddr_in);

		if ((s = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)) < 0) {
			perror("inet: socket");
			return - 1;
		}
		if (bind(s, &daddr, sizeof(addr)) < 0) {
			perror("bind");
			return - 1;
		}
		if (getsockname(s, &daddr, &dlen) != 0) {
			perror("inet: getsockname");
			(void)close(s);
			return - 1;
		}
		pmap_unset(ENTOMBPROG, ENTOMBVERS);
		pmap_set(ENTOMBPROG, ENTOMBVERS, IPPROTO_UDP,
		    ntohs(daddr.sin_port));
		if (dup2(s, 0) < 0) {
			perror("dup2");
			exit(1);
		}
	}
#endif	

	if ((transp = svcudp_create(0)) == NULL) {
		syslog(LOG_ERR|LOG_LOCAL0, "couldn't create udp transport");
		exit(1);
	}
	if (!svc_register(transp, ENTOMBPROG, ENTOMBVERS, server, 0)) {
		syslog(LOG_ERR|LOG_LOCAL0, "couldn't register ENTOMBPROG");
		exit(1);
	}

	/*
	 * Start serving
	 */
	svc_run();
	syslog(LOG_ERR|LOG_LOCAL0, "svc_run shouldn't have returned");
	abort();
}
#endif
