#include <stdio.h>
#include <syslog.h>
#include <netdb.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/wait.h>
#include <rpc/rpc.h>
#include "rpcstuff.h"
#include "rmtab.h"

/*
 * verify_caller -- make sure the the machine this request comes from
 *	is allowed to entomb on this filesystem.  We've already checked
 *	that the caller is root, but without this check anyone who had
 *	root on any other machine could entomb any file on our system.
 *
 *	We think a caller is ok if the filesystem he's trying to entomb
 *	on is mounted by the host the request comes from.  We read
 *	/etc/rmtab to decide if it's mounted.
 *
 *	verify_caller returns YES if the given address is verified for
 *	the given path.
 */
int
verify_caller(transp, ppchargs)
SVCXPRT *transp;
char **ppchargs;
{
	char *pchXHost, *pchXPath;
	struct rmtab *prm;
	int len = sizeof(struct in_addr);
	struct hostent *hep = NULL;
	struct sockaddr_in sin, *sinp;

	sinp = svc_getcaller(transp);
	hep = gethostbyaddr(&(sinp->sin_addr), sizeof(struct in_addr), AF_INET);
	if (NULL == hep) {
		fprintf(stderr, "verify: null hep, no slack.\n");
		return 0;
	}
	pchXHost = hep->h_name;
	pchXPath = ppchargs[1];

	while (NULL != (prm = getrmmach(pchXHost))) {
		if (str_ispref(prm->dir, pchXPath))
			return 1;
	}
	return 0;
}

/*
 * str_ispref -- returns 1 (YES) if the first argument is a prefix
 *	of the second.  Thus, str_ispref("/etc", "/etc/passwd") == 1.
 *	Returns  0 otherwise.
 */
int
str_ispref(pchpref, pchtest)
char *pchpref, *pchtest;
{
	while ('\0' != *pchpref && '\0' != *pchtest && *pchpref++ == *pchtest++)
		/*null*/;
	return '\0' == *pchpref;
}
