/*
** FsList -- make a list of filesystems to check for tombs.
**	The information comes from the CF config file and the
**	CRYPT #define.
*/
#ifndef lint
static char *rcsid= "$Id: fslist.c,v 1.11 88/11/11 01:19:27 mjb Exp $";
#endif
#include <stdio.h>
#include <strings.h>
#include <syslog.h>
#include <sys/file.h>
#include "tomb.h"
#include "fslist.h"

extern char *malloc();
char *sbpchTombDirs[NUMFS];	/* each of the form "/userd/tomb" */
char *sbpchDevs[NUMFS];		/* an array of like "/dev/ra2a"   */
int sbiMajorDevs[NUMFS];	/* the major device numbers	  */
int sbiMinorDevs[NUMFS];	/* the minor device numbers	  */
extern void FsList();

static int c = ' ';
#define GET() (c = getc(fp))
#define is_space(c) (' ' == c || '\n' == c || '\t' == c)
#define EAT() { while (is_space(c)) GET(); }
#define MUNCH() { do GET(); while ('\n' != c); }

/*
 * FsList - reads the config file and builds a list of the file systems to look
 * on for a user's file, and places the result in "TombDirs".  This list
 * is used later to display all of a user's entombed files, even if they're
 * on random filesystems.
 */
void
FsList()
{
	FILE *fp;
	static char buf[BUFSIZ];
	register int i, j;
	char sb[MAXFSNAMELEN];
	static int been_here = 0;
	
	/*
	 * it should only be necessary to get the filesystem list once
	 * per load.  That is, per invocation of rm, for instance.
	 */
	if (0 != been_here) {
		return;
	}
	been_here = 1;

	if (NULL == (fp = fopen(CF, "r"))) {
		syslog(LOG_ERR, "FsList: can't open %s: %m\n", CF);
		/*
 		 * I used to exit here, but that broke things when
		 * the machine was in single-user mode.
		 */
		sbpchTombDirs[0] = sbpchDevs[0] = NULL;
		sbiMajorDevs[0] = sbiMinorDevs[0] = -1;
		return;
	}
	setbuf(fp, buf);
	i = 0;
	EAT();
	for (i = 0; ! feof(fp); ++i) {
		if ('#' == c) {
			/* a comment */
			MUNCH();
			EAT();
			--i;
			continue;
		}
		/* a major,minor device number pair, followed by a 
		 * device name, followed by the filesystem name */
		/* first get the major and minor numbers */
		(void)  ungetc(c, fp);
		if (1 != (fscanf(fp, "%d", & sbiMajorDevs[i]))) {
			syslog(LOG_ERR, "fslist: hosed config file: majordev[%d]\n", i);
		}
		while (c != ',') GET();
		if (1 != (fscanf(fp, "%d", & sbiMinorDevs[i]))) {
			syslog(LOG_ERR, "fslist: hosed config file: minordev[%d]\n", i);
		}
		GET();
		EAT();
		/* now get the /dev/rzd0a filesystem name */
		for (j = 0; ! is_space(c); ++j) {
			sb[j] = c;
			GET();
		}
		sb[j] = '\000';
		sbpchDevs[i] = strcpy(malloc((unsigned)
			strlen(sb) + 1), sb);
		EAT();
		/* now get the /usera/ stuff */
		for (j = 0; ! is_space(c); ++j) {
			sb[j] = c;
			GET();
		}
		sb[j] = '\000';
		EAT();
		sbpchTombDirs[i] = strcpy(malloc((unsigned)
			strlen(sb) + 1), sb);
		/*
		 * if there are too many filesystems listed in my config
		 * file to put in the table, the overflow gets ignored,
		 * and we send a warning letter to the tomb owner
		 */
		if (NUMFS == i) {
			syslog(LOG_ERR, "fslist: too many filesystems in config file %s", CF);
			break;
		}
	}
	sbpchTombDirs[i] = sbpchDevs[i] = NULL;
	sbiMajorDevs[i] = sbiMinorDevs[i] = -1;
}
