/*
** these functions get called instead of the
** standard library functions.
**
** Copyright 1988 Purdue Research Foundation
*/
#ifndef lint
static char *rcsid = "$Id: tomb.c,v 1.11 88/10/15 18:29:44 mjb Exp $";
#endif

#include <stdio.h>
#include <sys/types.h>
#include <sys/param.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include "tomb.h"
#include "fslist.h"

extern char *getenv();

/*
 * is_env_set - checks to see if the environment variable is set, and 
 *	returns yes or no depending on whether we should entomb or not.
 */
static int
is_env_set()
{
	register char *pchEnVar;

	if (NULL != (pchEnVar = getenv("ENTOMB"))) {
		if (0 == strcasecmp(pchEnVar, "no")) {
			return 0;
		}
		if (0 == strcasecmp(pchEnVar, "false")) {
			return 0;
		}
		return 1;
	}
	/* set the default below: our default is to entomb. */
	return 1;
}

static int
tombexists(name)
	register char *name;
{
	register int i;
	struct stat statbuf;

#if defined S_IFLNK
	if (-1 == lstat(name, & statbuf)) {
#else
	if (-1 == stat(name, & statbuf)) {
#endif
		/*
		 * if the file doesn't exist, or something of this nature,
		 * we don't need to fork.  We'll say the tomb doesn't exist.
		 */
		return 0;
	}

	/*
	 * if it's on an nfs, we need to look elsewhere for the tomb,
	 * so we'll say it exists for now.
	 */
	if (NFS_MAJ_DEV == major(statbuf.st_dev))
		return 1;

	FsList();
	for (i = 0; major(statbuf.st_dev) != sbiMajorDevs[i]
		|| minor(statbuf.st_dev) != sbiMinorDevs[i]; ++i) {
		if (-1 == sbiMajorDevs[i]) {
			return 0;
		}
	}
	return 1;
}

static void
entomb(what2do, path)
	char *what2do, *path;
{
	/*
	 * if there's no tomb, we don't need to fork.
	 */
	if (! tombexists(path))
		return;

	if (vfork() == 0) {
		/*
		 * ENTOMB is a set-uid program whose name is passed via
		 * command-line option to cc.
		 */
		(void) execl(ENTOMB, what2do, path, (char *)0);
		perror(ENTOMB);
		_exit(1);
	} else {
		(void) wait((union wait *)0);
		return;
	}
	return;
}

int
creat(name, mode)
	char *name;
	int mode;
{
	if (0 == access(name, F_OK) && is_env_set())
		entomb("copy", name);

	return creat_r(name, mode);
}

int
open(file, flags, mode)
	char *file;
	int flags, mode;
{
	struct stat statbuf;

	if ((flags&(O_CREAT|O_TRUNC)) && 0 == access(file, F_OK) && is_env_set()) {
		entomb("copy", file);
	}
	return open_r(file, flags, mode);
}

int
rename(from, to)
	char *from, *to;
{
	struct stat statbuf;
	struct stat statbuf2;

	if (! is_env_set()) {
		return rename_r(from, to);
	}

#ifdef S_IFLNK
	if (0 == lstat(to, &statbuf) && 0 == lstat(from, &statbuf2)) {
#else
	if (0 == stat(to, &statbuf) && 0 == stat(from, &statbuf2)) {
#endif S_IFLNK
		if (S_IFREG == (statbuf.st_mode&S_IFMT) && statbuf.st_dev == statbuf2.st_dev ) {
			entomb("rename", to);
		}
	} 
	return rename_r(from, to);
}

int
truncate(path, length)
	char *path;
	int length;
{
	struct stat statbuf;

	if (! is_env_set())
		return truncate_r(path, length);

	/*
	 * What should probably be done here is to make a copy of the
	 * file in the tomb area and then call the real truncate.  For
	 * now, we are going to assume that the user of truncate is a 
	 * knowledgeable Joe and only back up the file if he is truncating
	 * to zero length.
	 */

	if (0 == length && 0 == access(path, F_OK))
		entomb("copy", path);
	return truncate_r(path, length);
}

int
unlink(name)
	char *name;
{
	if (! is_env_set())
		   return unlink_r(name);

	entomb("rename", name);
	return unlink_r(name);
}
