/*
 * the main loop for the preendaemon.  This code could all run as the owner
 * of the tomb, except that it needs read permission on the raw disk device
 * in how_full.
 */
#ifndef lint
static char *rcsid="$Id :$";
#endif

#include <stdio.h>
#include <strings.h>
#include <fstab.h>
#include <signal.h>
#include <setjmp.h>
#include <syslog.h>
#include <pwd.h>
#include <sys/ioctl.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/file.h>
#include <sys/stat.h>
#include "tomb.h"
#include "fslist.h"
#include "preentomb.h"
#include "main.h"

static jmp_buf main_env;

int
PreenD()
{
	int i;

	/*
	 * If we're running as root, drop down to charon.
	 */
	if (0 == getuid()) {
		struct passwd *pwent;
		if (NULL == (pwent = getpwnam(TOMBID))) {
			fprintf(stderr, "preend: can't get %s's passwd entry\n", TOMBID);
			exit(2);
		}
		(void) setuid(pwent->pw_uid);
	}

	openlog("preend", 0, LOG_DAEMON);

	if (! Debug)
		daemonize();
	
	if (Debug) {
		printf("Locating tomb directories\n");
	}
	FsList();

	(void) setjmp(main_env);
	Read_cf();
		
	/*
	 * now sit in an infinite loop an work
	 */
	for (;;) {
		/* 
		 * Check filesystem fill percentages and return
		 * an indication of how severely we need to trim this
		 * tomb
		 */
		for(i = 0; NULL != sbpchTombDirs[i]; i++) {
			/* 
			 * Actually preen the given tomb 
			 */
			if (Debug) {
				fprintf(stderr, "Cleaning %s\n",
					sbpchTombDirs[i]);
			}
			preenfs(sbpchDevs[i], sbpchTombDirs[i]);
		}
		if (Debug)
			printf("sleeping...\n");
		(void) sleep((unsigned) sleeptime * 60);
	}
}

/*
 * finish - end the preend.  a pointer to this function is given to
 *	signal() for those signals we would like to kill us.
 */
finish()
{
	(void) signal(SIGTERM, SIG_IGN);
	if (Debug) {
		printf("preend: exiting\n");
	}
	exit(0);
}

/*
 * hup_handler - jump back to the beginning of the main routine, where
 * 	we read the config file again.
 */
static void
hup_handler()
{
	syslog(LOG_INFO, "caught a HUP");
	(void) longjmp(main_env, 0);
	/*NOTREACHED*/
}

daemonize()
{
	int res, td;

	(void) signal(SIGHUP,  hup_handler);	
	(void) signal(SIGQUIT, SIG_IGN);
	(void) signal(SIGINT,  SIG_IGN);
#if defined(BSD4_2) || defined(BSD4_3)
	(void) signal(SIGTTOU, SIG_IGN);
	(void) signal(SIGTSTP, SIG_IGN);
#endif BSD4_2 || BSD4_3
	(void) signal(SIGTERM, finish);

	(void) chdir("/");

	if (FAIL == (res = fork())) {
		(void) fprintf(stderr, "Preend: couldn't start\n");
		perror("fork");
		exit(1);
	}
	if (0 != res) {			  /* if the parent */
		exit(0);
	}

	/*
	 * lose our controlling terminal
	 */

#ifdef BSD2_9
	td = open("/dev/tty", O_RDWR);
#else BSD2_9
	td = open("/dev/tty", O_RDWR, 0600);
#endif BSD2_9

	if (td >= 0) {
		(void) ioctl(td, TIOCNOTTY, (char *)NULL);
	}

	/* 
	 * Further disassociate this process from the terminal 
	 * Maybe this will allow you to start a daemon from rsh,
	 * i.e. with no controlling terminal.
	 */

	(void) setpgrp(0,getpid());
}
