#ifndef lint
static char *rcsid= "$Id: preentomb.c,v 1.9 88/09/25 12:20:20 mjb Exp $";
#endif
#include <stdio.h>
#include <ctype.h>
#include <syslog.h>
#include <sys/types.h>
#include <sys/dir.h>
#include <sys/stat.h>
#include <sys/time.h>
#include "capacity.h"
#include "survives.h"
#include "main.h"

extern char *rindex();
extern long atol();
extern void preenudir();
extern int errno;
extern char *sys_errlist[];

long tNow;			/* used externally */

/*
 * preenfs -- preen the tomb directory on a particular filesystem.
 *	we're given the filesystem name with the CRYPT appended, and
 *	also a slash. e.g: preenfs("/usera/tomb/", 94)
 */
void
preenfs(pchDev, pchTombDir)
	char *pchDev;			/* the raw disk device involved */
	char *pchTombDir;		/* the name of the tomb directory */
{
	register DIR *pD;		/* directory being preened	*/
	register struct direct *pd;	/* current directory entry	*/
	struct timeval tp;		/* figure out what time it is	*/
	struct timezone tzp;		/* ... same			*/

	if (-1 == gettimeofday(&tp, &tzp)) {
		syslog(LOG_ERR|LOG_DAEMON, "gettimeofday: %m");
		return;
	}
	tNow = tp.tv_sec; 

	if (Debug && NULL != pchTombDir) {
		fprintf(stderr, "chdir %s\n", pchTombDir);
	}
	if (-1 == chdir(pchTombDir)) {
		syslog(LOG_ERR|LOG_DAEMON, "preenfs: can't chdir to %s: %m", pchTombDir);
		return;
	}
	if (NULL == (pD = opendir("."))) {
		syslog(LOG_ERR|LOG_DAEMON, "opendir: %s: %m", pchTombDir);
		return;
	}
	
	/*
	 * we read in a list of files, which we expect to be subdirectories,
	 * each a uid, full of files entombed by that user.
	 */
	for (pd = readdir(pD); pd != NULL; pd = readdir(pD)) {
		if (strcmp(pd->d_name, ".") == 0)
			continue;
		if (strcmp(pd->d_name, "..") == 0)
			continue;
		/*
		 * preen this user's directory.  First we preen away the
		 * files like "core", so that perhaps important files can
		 * stay entombed longer.  Then we preen the other files.
		 */
		if (Debug)
			fprintf(stderr, "preening worthless\n");
		preenudir(pchDev, pchTombDir, pd->d_name, dink_worthless, 0);
		if (Debug)
			fprintf(stderr, "preening others\n");
		preenudir(pchDev, pchTombDir, pd->d_name, dink_other, 1);
		(void) rmdir(pd->d_name);
	}
	closedir(pD);
	if (Debug)
		fprintf(stderr, "chdir ..\n");
	if (-1 == chdir("..")) {	/* back to where we started */
		syslog(LOG_ERR|LOG_DAEMON, "chdir .. failed: %m");
	}
}

/*
 * preenudir -- preen a user's tomb directory, like /usera/tomb/mjb.  The
 * function, pfPreen, should be called with information about a file; it
 * returns a flag telling us whether to dink the file or not.
 */
static void
preenudir(pchDev, pchTombDir, pchUDir, pfPreen, flog)
	char *pchDev, *pchTombDir, *pchUDir;
	int (*pfPreen)();
	int flog;			/* to log or not		*/
{
	register DIR *pD;		/* directory being preened	*/
	register struct direct *pd;	/* current directory entry	*/
	int fDink;			/* flag: dink this file?	*/
	char *pch;			/* pointer into name		*/
	struct stat statb;		/* handy buffer for stat	*/
	long tombTime;			/* time entombed		*/
	int iHowFull;			/* the fullness of this fs	*/

	if (Debug && NULL != pchTombDir) {
		fprintf(stderr, "chdir %s\n", pchUDir);
	}
	if (-1 == chdir(pchUDir)) {
		fprintf(stderr, "preenudir: can't chdir to %s: %m", pchUDir);
		return;
	}
	if (NULL == (pD = opendir("."))) {
		syslog(LOG_ERR|LOG_DAEMON, "opendir %s: %m", pchUDir);
		return;
	}
	iHowFull = how_full(pchDev);
	for (pd = readdir(pD); pd != NULL; pd = readdir(pD)) {
		if (strcmp(pd->d_name, ".") == 0)
			continue;
		if (strcmp(pd->d_name, "..") == 0)
			continue;
		if (stat(pd->d_name, & statb) < 0 ) {
			syslog(LOG_ERR|LOG_DAEMON, "stat %s/%s: %m", pchTombDir, pd->d_name);
			continue;
		}

		/* 
		 * find the last @, and get the time that the file was 
		 * entombed.
		 */
		pch = rindex(pd->d_name,'@');
		if (NULL == pch)
			continue;
		if (0 == (tombTime = atol(pch+1))) {
			continue;
		}
		if (! (*pfPreen)(pd->d_name, statb.st_size, tombTime, iHowFull))
			continue;
		/*
		 * we're going to dink this one
		 */

		if (Debug)
			printf("unlinking %s/%s/%s\n", pchTombDir, pchUDir, pd->d_name);
#ifdef LOGFILE
		if (flog) {
			int h, m, s;
			long age;
			age = tNow - tombTime;
			h = age/3600;
			m = age%3600 / 60;
			s = age%3600%60;

			syslog(LOG_INFO|LOG_DAEMON, "%s age %02d:%02d:%02d full %d size %d", pd->d_name, h, m, s, iHowFull, (int) statb.st_size);
		}
#endif LOGFILE
		
#ifndef DEBUG
		if (-1 == unlink(pd->d_name))
			syslog(LOG_ERR|LOG_DAEMON, "unlink: %s: %m", pd->d_name);
#endif
	}
	if (Debug)
		fprintf(stderr, "chdir ..\n");
	if (-1 == chdir("..")) {
		syslog(LOG_ERR|LOG_DAEMON, "preenudir: can't chdir to ..: %m\n" );
		return;
	}
	closedir(pD);
}
