/*
 * code to decide whether a given file gets dinked or not
 */
#include <stdio.h>
#include <ctype.h>
#include "preentomb.h"
#include "survives.h"
#include "main.h"
extern char *rindex();
/*
 * specify these cutoffs:
 *	
 *	Adjust these numbers and thresholds to provide the performance you
 *	want.
 */

/* variables set from the config file */
static int iFull1, iFull2, iFull3;
long tTime1, tTime2, tMinTime;
double dSecPerByte;

#define YES 1
#define NO  0

extern long tNow;

/*
 * preen_other - deciding whether a file should be removed yet, according to
 *	its name, its size, and the fullness of its filesystem.  Returns
 *	YES or NO.  Also uses the external tNow, the time it is "now".
 */
int
dink_other(pchName, lSize, tDate, iFullness)
	char *pchName;			/* basename of the entombed file */
	long lSize;			/* in bytes */
	long tDate;			/* creation date */
	int iFullness;			/* as a percent */
{
	long age, lifespan;

#ifdef DEBUG
fprintf(stderr, "dink_other(%s, %ld, %ld, %d)\n", pchName, lSize, tDate, iFullness);
#endif

	age = tNow - tDate;

	if (iFullness > FULL_3)
		return YES;

	if (iFullness > FULL_2) {
		if (age <= MINTIME)
			return NO;
		return YES;
	}

	lifespan = TIME_2 - ((double) SECPERBYTE * (double) lSize);

	if (iFullness > FULL_1) {
		if (age <= MINTIME)
			return NO;
		if (age > lifespan)
			return YES;
		return NO;
	}
	if (age <= MINTIME)
		return NO;
	if (age <= TIME_1)
		return NO;
	return YES;
}

/*
 * dink_worthless - check the given file name to see if we should dink it,
 *	based only on it's age and it's name.  So the fact that
 * 	worthless files only survive MINTIME is determined here.
 */
int
dink_worthless(pchName, lSize, tDate, iFullness)
	char *pchName;
	long lSize, tDate;
	int iFullness;
{
	char *pchAt;		/* pointer to the '@' sign		*/
	char *pchDot;		/* pointer to before the extension	*/
	int fIsIt;		/* a temporary returned at the end	*/

#ifdef DEBUG
fprintf(stderr, "dink_worthless(%s, %ld, %ld, %d)\n", pchName, lSize, tDate, iFullness);
#endif
	/* if it's less than MINTIME old, it survives. */

	if (tNow - tDate < MINTIME)
		return NO;

	/* is the file in the wrong format?  Something is wrong, so I'll
 	 * be conservative.
	 */
	if (NULL == (pchAt = rindex(pchName, '@'))) {
		return NO;
	}
	*pchAt = '\000';

	/* can you say GOTO?  I knew you could.	 This loops exactly once. */
	for (;;) {			
		if ('#' == *pchName) {
			fIsIt = YES;
			break;
		}
		if ('~' == *(pchName + strlen(pchName) - 1)) {
			fIsIt = YES;
			break;
		}
		if (0 == strcmp(pchName, "core")) {
			fIsIt = YES;
			break;
		}
		if (0 == strcmp(pchName, "a.out")) {
			fIsIt = YES;
			break;
		}
		/*
		 * for the rest we're only concerned with the extension.
		 * filenames w/o extensions aren't worthless.
		 */
		if (NULL == (pchDot = rindex(pchName, '.'))) {
			fIsIt = NO;
			break;
		}
		if (0 == strcmp(pchDot, ".o")) {
			fIsIt = YES;
			break;
		}
		if (0 == strcmp(pchDot, ".bak")) {
			fIsIt = YES;
			break;
		}
		if (0 == strcmp(pchDot, ".dbg")) {
			fIsIt = YES;
			break;
		}
		fIsIt = NO;
		break;
	}
	*pchAt = '@';
	return fIsIt;
}

static void
parse_err()
{
	fprintf(stderr, "parse_err in %s\n", CF);
	exit(3);
}

/*
 * Read_cf -- call this routine upon startup and upon receipt of a signal
 * 	to cause the preend config file to be read.  If the file can't be
 *	found, defaults are assigned.  Defaults are also assigned to those
 *	variables that can't be read from the config file.  The name of the
 *	config file is given to us as a #define: "CF"
 */
#define GET() (c = getc(pF))
#define EAT() while (isspace(c) && EOF != c) GET();
#define COMMENTCHAR ('#')
void
Read_cf()
{
	register int c = ' ', i;
	FILE *pF;
	/*
	 * these have to be longer than the longest variable name
	 */
	static char sbVariable[25], sbValue[25];

	iFull1 = FULL_1;
	iFull2 = FULL_2;
	iFull3 = FULL_3;
	tTime1 = TIME_1;
	tTime2 = TIME_2;
	tMinTime = MINTIME;
	dSecPerByte = SECPERBYTE;

	if (NULL == (pF = fopen(CF, "r"))) {
		perror(CF);
		return;
	}
	/*
	 * we read the entire config file, a pair of tokens at a time.
	 */
	for (;;) {
		EAT();
		if (EOF == c)
			break;

		if (COMMENTCHAR == c)
			while ('\n' != c && EOF != c)
				GET();
		i = 0;
		while (! isspace(c) && EOF != c)
			sbVariable[i++] = c, GET();
		sbVariable[i] = '\000';
		i = 0;
		EAT();
		while (! isspace(c) && EOF != c)
			sbValue[i++] = c, GET();
		i = '\000';

		if (0 == strcmp(sbVariable, "FULL_1")) {
			if (1 != sscanf(sbValue, "%d", & iFull1)) {
				parse_err();
				return;
			}
			if (Debug)
				printf("FULL_1: %d\n", iFull1);
			continue;
		}
		if (0 == strcmp(sbVariable, "FULL_2")) {
			if (1 != sscanf(sbValue, "%d", & iFull2)) {
				parse_err();
				return;
			}
			if (Debug)
				printf("FULL_2: %d\n", iFull2);
			continue;
		}
		if (0 == strcmp(sbVariable, "FULL_3")) {
			if (1 != sscanf(sbValue, "%d", & iFull3)) {
				parse_err();
				return;
			}
			if (Debug)
				printf("FULL_3: %d\n", iFull3);
			continue;
		}
		if (0 == strcmp(sbVariable, "TIME_1")) {
			if (1 != sscanf(sbValue, "%ld", & tTime1)) {
				parse_err();
				return;
			}
			if (Debug)
				printf("TIME_1: %d\n", tTime1);
			continue;
		}
		if (0 == strcmp(sbVariable, "TIME_2")) {
			if (1 != sscanf(sbValue, "%ld", & tTime2)) {
				parse_err();
				return;
			}
			if (Debug)
				printf("TIME_2: %d\n", tTime2);
			continue;
		}
		if (0 == strcmp(sbVariable, "MINTIME")) {
			if (1 != sscanf(sbValue, "%ld", & tMinTime)) {
				parse_err();
				return;
			}
			if (Debug)
				printf("MINTIME: %ld\n", tMinTime);
			continue;
		}
		if (0 == strcmp(sbVariable, "SECPERBYTE")) {
			if (1 != sscanf(sbValue, "%lf", & dSecPerByte)) {
				parse_err();
				return;
			}
			if (Debug)
				printf("SECPERBYTE: %f\n", dSecPerByte);
			continue;
		}
	}
}
