/*
**  build a linked list of all the files entombed for this user.
**  use the table of filesystem names generated by "fslist." 
*/
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/dir.h>
#include <sys/param.h>
#include <pwd.h>
#include <strings.h>
#include "fslist.h"
#include "lists.h"
#include "init.h"

extern char *malloc();
extern long atol();

/* functions defined in and local to this file			*/
extern time_t remove_date();
extern int datecmp();

char *pchUid;
int iUid, iTombUid;
int iGid, iTombGid;
LE *pLEFilesList = nilLE;

void
Init()
{
	register int i;
	DIR *pD;
	struct direct *pdEntry;
	struct passwd *pwent;
	static char sbUTombDir[MAXPATHLEN];
	static char sbTombedFile[MAXPATHLEN+MAXNAMLEN+1], *pchFileStart;
	LE *pLE;
	static int init_already = 0;
	struct stat stat_buf;

	if (init_already)
		return;
	init_already = 1;

	/* find the user-id of the person who's running this program	*/
	if (NULL == (pwent = getpwuid(iUid = getuid()))) {
		fprintf(stderr, "getpwuid: can't get your accounting information\n");
		exit(2);
	}
	pchUid = pwent->pw_name;
	iGid = getgid();
	iTombUid = geteuid();
	iTombGid = getegid();

	/* get the list of filesystems		*/
	FsList();		
	
	/*
	 * zip through the array of filesystem names, opening our tomb
	 * directory in each, and added the entombed files we find there to
	 * the linked list.
	 */
	for (i = 0; NULL != sbpchTombDirs[i]; ++i) {
		(void) strcpy(sbUTombDir, sbpchTombDirs[i]);
		(void) strcat(sbUTombDir, "/");
		(void) strcat(sbUTombDir, pchUid);
		if (NULL == (pD = opendir(sbUTombDir))) {
			/* the user may have removed no files on this filesys*/
			continue;
		}
		(void) strcpy(sbTombedFile, sbUTombDir);
		(void) strcat(sbTombedFile, "/");
		pchFileStart = sbTombedFile + strlen(sbTombedFile);

		while (NULL != (pdEntry = readdir(pD))) {
			sbUTombDir[0] = '\000';
			/* ignore deleted files */
			if (0 == pdEntry->d_ino)
				continue;
			if (0 == strcmp("..", pdEntry->d_name))
				continue;
			if (0 == strcmp(".", pdEntry->d_name))
				continue;
			(void) strcat(sbUTombDir, pdEntry->d_name);
			(void) strcpy(pchFileStart, pdEntry->d_name);
			if (stat(sbTombedFile, &stat_buf) < 0) {
				fprintf(stderr, "stat: ");
				perror(sbTombedFile);
			}

			pLE = newLE();
			pLE->next = nilLE;
			pLE->fs = i;
			pLE->size = stat_buf.st_size;
			pLE->date = remove_date(sbUTombDir);
			pLE->name = malloc((unsigned)strlen(sbUTombDir)+1);
			if (NULL == pLE->name)
				exit(12);	/*ENOMEM*/
			(void) strcpy(pLE->name, sbUTombDir);

			SLInsert(& pLEFilesList, pLE, datecmp);
		}
	}
}

static char *my_pchFile;
static int
namecmp(pLE)
	LE *pLE;
{
	return strcmp(pLE->name, my_pchFile);
}

static int
datecmp(pLElist, pLEelem)
	LE *pLElist, *pLEelem;
{
	return pLEelem->date - pLElist->date;
}

/*
 * real_name - for use [for instance] when joe user types  "restore foo"
 *	to translate the "foo" into "/usera/tomb/joe/foo@23432423".  We
 *	scan the list of entombed files, pLEFilesList, and look for the
 *	right one.  We put the name in a buffer that is declared here,
 *	and return a pointer to it, or NULL if there is no such entombed
 * 	file.
 */
char *
real_name(pchFile)
	char *pchFile;
{
	static char sbFile[MAXPATHLEN+MAXNAMLEN+1];
	LE *pLE;

	my_pchFile = pchFile;
	pLE = Retrieve(pLEFilesList, namecmp);
	if (NULL == pLE) {
		return NULL;
	}
	(void) strcpy(sbFile, sbpchTombDirs[pLE->fs]);
	(void) strcat(sbFile, "/");
	(void) strcat(sbFile, pchUid);
	(void) strcat(sbFile, "/");
	(void) strcat(sbFile, pLE->name);
	(void) strcat(sbFile, "@");
	(void) sprintf(sbFile+strlen(sbFile), "%d", pLE->date);
	return sbFile;
}

/*
 * remove_date - takes a file name of the form <file>@<numericdate>,
 *	replaces the '@' with \0, and returns the time_t associated
 *	with <numericdate>.
 */
static time_t
remove_date(pchfile)
	char *pchfile;
{
	char *pch;

	if (NULL == (pch = rindex(pchfile, '@'))) {
		/* return Jan 1 1970 for mis-formed filenames	*/
		return 0;
	}
	*pch = '\000';
	return (time_t) atol(pch + 1);
}
