/*
** interact -- everything (almost) we need to do interactive
**	commands for "unrm".  Uses Steve Uitti's libcmd.
**	Expects "CF" to be defined as the entomb config file
**	at compile time.
**
**	Matthew Bradburn, PUCC UNIX Group
*/
#ifndef lint
static char *Rcsid= "$Id: interact.c,v 1.4 88/08/04 14:18:06 mjb Exp $";
#endif
#include <stdio.h>
#include <sys/param.h>
#include <strings.h>
#include <cmd.h>
#include "unrm.h"
#include "restore.h"
#include "lists.h"
#include "changedir.h"
#include "init.h"
#include "interact.h"
#include "listtombs.h"
#include "ls.h"
#include "purge.h"
#include "show.h"
#include "restore.h"

#define CMDLINELEN	256		/* maximum length user can type	*/

extern char *getwd();

static char *ppchCmds[] = {
	"bye \t\t exit this program\n",
	"cd \t\t change working directory of this program\n",
	"list \t\t list the files currently entombed\n",
	"ls \t\t list the files in the working directory\n",
	"purge \t\t remove files from the tomb\n",
	"pwd \t\t print the path to the working directory\n",
	"quit \t\t exit this program\n",
	"restore \t restore a file to the working directory\n",
	"show \t\t invokes the user's pager on a file in the tomb\n",
	"verbose \t toggles verbose mode\n",
	"help \t\t print help information\n",
	"? \t\t print help information\n",
	NULL
};

/*
 * these have to match in number and order the elements of the above array
 */
#define BYE	(0)
#define CD 	(1)
#define LIST 	(2)
#define LS	(3)
#define PURGE	(4)
#define PWD	(5)
#define QUIT	(6)
#define RESTORE	(7)
#define SHOW	(8)
#define VERBOSE	(9)
#define HELP   (10)
#define QUESTY (11)

/*
 * to be called when no arguments are given to "unrm".  enters
 * interactive mode, and stays there.
 */
void
Interactive()
{
	static char cmdbuf[CMDLINELEN];
	static char sb[MAXPATHLEN];
	auto char *pch, *pchCmd;

	if (Verbose) {
		printf("This program may be used to restore deleted files: type \"help\" for help\n");
	}
	for (;;) {
		/* print a prompt */
		printf("%s> ", progname);
		(void) fflush(stdout);

		if (NULL == fgets(cmdbuf, CMDLINELEN, stdin)) {
			(void) putchar('\n');
			exit(0);
		}
		if (NULL != (pch = index(cmdbuf, '\n')))
			*pch = '\000';

		switch (cmdprs(cmdbuf, ppchCmds)) {
		case NULLINP:
			/* prompt again */
			break;
		case AMBIGINP:
			fprintf(stderr, "ambiguous command\npossibilities:\n");
			cmdamb(cmdbuf, ppchCmds, stdout);
			break;
		case UNRECINP:
			fprintf(stderr, "unknown command\n");
			break;
		case CD:
			/*
			 * change this program's current working directory
			 */
			ChangeDir(cmddlm(cmdbuf));
			break;
		case LIST:
			/*
			 * list files belonging to the user in each
			 * tomb directory
			 */
			ListTombs();
			break;
		case LS:
			/*
			 * print the files in the current working directory
			 */
			Ls(cmdbuf);
			break;
		case PURGE:
			/*
			 * remove a file or files from the tomb directory
			 */
			Purge(cmddlm(cmdbuf));
			break;
		case PWD:
			/*
			 * print this program's current working directory
			 */
			if (-1 == setreuid(iTombUid, iUid)) {
				fprintf(stderr, "pwd: can't change uid\n");
				perror("setreuid");
				exit(1);
			}
			if (-1 == setregid(iTombGid, iGid)) {
				fprintf(stderr, "pwd: can't change gid\n");
				perror("setregid");
				exit(1);
			}
			if (0 != getwd(sb)) {
				(void) puts(sb);
			} else {
				fprintf(stderr, "getwd failed: %s\n", sb);
				exit(2);
			}
			if (-1 == setreuid(iUid, iTombUid)) {
				fprintf(stderr, "pwd: can't change uid back\n");
				perror("setreuid");
				exit(1);
			}
			if (-1 == setregid(iGid, iTombGid)) {
				fprintf(stderr, "pwd: can't change gid back\n");
				perror("setregid");
				exit(1);
			}
			break;
		case BYE:		/* just cuz yuf wanted it */
			/*FALLTHROUGH*/
		case QUIT:
			/*
			 * leave this program
			 */
			exit(0);
			break;
		case RESTORE:
			/*
			 * unremove the given file into the current
			 * working directory and remove it from the 
			 * tomb directory
			 */
			 Restore(cmddlm(cmdbuf));
			 break;
		case SHOW:
			/*
			 * invoke the user's pager on the given file
			 * in the tomb directory
			 */
			 Show(cmddlm(cmdbuf));
			 break;
		case VERBOSE:
			/*
			 * toggle verbose mode
			 */
			Verbose = ! Verbose;
			break;
		case HELP:
			/*FALLTHROUGH*/
		case QUESTY:
			/*
			 * if there was only "help" on the command line, print
			 * a list of commands.  If there was more, print help
			 * for that command.
			 */
			pchCmd = cmddlm(cmdbuf);
			if ('\000' == *pchCmd) {
				printf("Commands may be abbreviated.  Commands are:\n");
				if (Verbose) {
					cmdlst(ppchCmds, stdout);
				} else {
					cmdshl(ppchCmds, stdout);
				}
				(void) putchar('\n');
				break;
			}
			cmdhlp(cmdbuf, ppchCmds, stdout);
			break;
		default:
			fprintf(stderr, "Internal error.\n");
			break;
		}
	}
}
