#include <stdio.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <cmd.h>
#include "tomb.h"
#include "unrm.h"
#include "restore.h"
#include "lists.h"
#include "init.h"

static char *my_pchArg;		/* a copy of pchArg, used in namecmp */

static int
namecmp(pLE)
	LE *pLE;
{
	return strcmp(pLE->name, my_pchArg);
}

/*
 * Restore - restore an entombed file, and delete it from the tomb.
 *	the pchFile argument is like "foo", for the examples, and may
 *	have trailing, but not leading, whitespace.  Modified to accept
 *	"foo baz bar"
 */

void
Restore(pchArg)
	char *pchArg;
{
	FILE *pFin, *pFout;
	char answer[5];		/* enough room for 'yes\n'		*/
	char *pchNext;		/* the next file in the list to restore */
	char *pchFile;		/* the full file name */
	LE *pLE;
	int c;
	int Mode;		/* the modes to restore the file to 	*/
	struct stat stat_buf;	/* how to get the modes			*/

	pchNext = cmddlm(pchArg);
	*cmdscm(pchArg) = '\000';
	my_pchArg = pchArg;

	if ('\000' == *pchArg) {
		printf("usage: restore <file> ...\n");
		return;
	}
	if (0 == strcmp("*", pchArg)) {
		while (NULL != pLEFilesList) {
			pLE = pLEFilesList->next;
			Restore(pLEFilesList->name);
			pLEFilesList = pLE;
		}
		return;
	}
	if (NULL == (pchFile = real_name(pchArg))) {
		printf("%s: no such file\n", pchArg);
		return;
	}
		
	/*
	 * open the entombed file as charon
	 */
	if (NULL == (pFin = fopen(pchFile, "r"))) {
		perror(pchFile);
		return;
	}
	/* find its permissions */
	if (fstat(fileno(pFin), & stat_buf) < 0) {
		perror(pchFile);
		return;
	}
	Mode = stat_buf.st_mode & 0777;
		
	/*
	 * open the output file as joe user
	 */
	if (FAIL == setreuid(iTombUid, iUid)) {
		perror("setreuid");
		exit(1);
	}
	if (FAIL == setregid(iTombGid, iGid)) {
		perror("setregid");
		exit(1);
	}
	if (fexists(pchArg)) {
		printf("%s exists: overwrite? ", pchArg);
		(void) fgets(answer, sizeof(answer), stdin);
		clearerr(stdin);
		if ('y' != answer[0] && 'Y' != answer[0]) {
			if (EOF == fclose(pFin))
				perror("fclose");
			if (FAIL == setreuid(iUid, iTombUid)) {
				perror("setreuid");
				exit(1);
			}
			if (FAIL == setregid(iGid, iTombGid)) {
				perror("setregid");
				exit(1);
			}
			return;
		}
	}
	if (NULL == (pFout = fopen(pchArg, "w"))) {
		perror(pchArg);
		if (FAIL == setreuid(iUid, iTombUid)) {
			perror("setreuid");
			exit(1);
		}
		if (FAIL == setregid(iGid, iTombGid)) {
			perror("setregid");
			exit(1);
		}
		return;
	}
	if (-1 == fchmod(fileno(pFout), Mode))
		perror("fchmod");
	while (EOF != (c = getc(pFin)))
		(void) putc(c, pFout);
	(void) fclose(pFin);
	(void) fclose(pFout);
	if (FAIL == setreuid(iUid, iTombUid)) {
		perror("setreuid");
		exit(1);
	}
	if (FAIL == setregid(iGid, iTombGid)) {
		perror("setregid");
		exit(1);
	}
	if (FAIL == unlink(pchFile)) {
		perror(pchFile);
		exit(1);
	}

	if (nilLE == Delete(& pLEFilesList, namecmp))
		printf("internal error in SLDelete\n");

	/* take care of the rest of the arguments */
	if ('\000' != *pchNext)
		Restore(pchNext);
	return;
}

/*
 * fexists - tries to open the given file for read, to see if it
 *	exists or not.  Returns 0 or 1.
 */
static int
fexists(pchFile)
	char *pchFile;
{
	FILE *pF;

	if (NULL == (pF = fopen(pchFile, "r")))
		return 0;
	(void) fclose(pF);
	return 1;
}
